@@ -46,7 +46,6 @@
 #include <telepathy-glib/svc-generic.h>
 #include <telepathy-glib/util.h>
 
-#include "channel-utils.h"
 #include "mcd-account-priv.h"
 #include "mcd-channel-priv.h"
 #include "mcd-enum-types.h"
@@ -80,7 +79,6 @@ struct _McdChannelPrivate
 
     McdChannelRequestData *request_data;
     GList *satisfied_requests;
-    gint64 latest_request_time;
 };
 
 struct _McdChannelRequestData
@@ -178,10 +176,7 @@ on_members_changed (TpChannel *proxy, const gchar *message,
     }
 
     if (removed && removed->len > 0 &&
-        (actor == 0 ||
-         reason == TP_CHANNEL_GROUP_CHANGE_REASON_ERROR ||
-         (actor != self_handle && actor != conn_self_handle) ||
-         reason == TP_CHANNEL_GROUP_CHANGE_REASON_NO_ANSWER))
+        (actor == 0 || (actor != self_handle && actor != conn_self_handle)))
     {
         for (i = 0; i < removed->len; i++)
         {
@@ -253,12 +248,53 @@ on_channel_ready (TpChannel *tp_chan, const GError *error, gpointer user_data)
 	_mcd_channel_setup_group (channel);
 }
 
+static gboolean
+mcd_channel_should_close (McdChannel *channel,
+                          const gchar *verb)
+{
+    McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
+    const GError *invalidated;
+    GQuark channel_type;
+
+    if (priv->tp_chan == NULL)
+    {
+        DEBUG ("Not %s %p: no TpChannel", verb, channel);
+        return FALSE;
+    }
+
+    invalidated = TP_PROXY (priv->tp_chan)->invalidated;
+
+    if (invalidated != NULL)
+    {
+        DEBUG ("Not %s %p, already invalidated: %s %d: %s",
+               verb, channel, g_quark_to_string (invalidated->domain),
+               invalidated->code, invalidated->message);
+        return FALSE;
+    }
+
+    channel_type = tp_channel_get_channel_type_id (priv->tp_chan);
+
+    if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_CONTACT_LIST)
+    {
+        DEBUG ("Not %s %p, it's a ContactList", verb, channel);
+        return FALSE;
+    }
+
+    if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_TUBES)
+    {
+        DEBUG ("Not %s %p, it's an old Tubes channel", verb, channel);
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
 void
 _mcd_channel_close (McdChannel *channel)
 {
     McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
 
-    if (!_mcd_tp_channel_should_close (priv->tp_chan, "closing"))
+    if (!mcd_channel_should_close (channel, "closing"))
     {
         return;
     }
@@ -273,7 +309,7 @@ _mcd_channel_undispatchable (McdChannel *channel)
 {
     McdChannelPrivate *priv = MCD_CHANNEL_PRIV (channel);
 
-    if (!_mcd_tp_channel_should_close (priv->tp_chan, "destroying"))
+    if (!mcd_channel_should_close (channel, "destroying"))
     {
         return;
     }
@@ -1025,6 +1061,61 @@ _mcd_channel_get_immutable_properties (McdChannel *channel)
     return ret;
 }
 
+/*
+ * _mcd_channel_details_build_from_list:
+ * @channels: a #GList of #McdChannel elements.
+ *
+ * Returns: a #GPtrArray of Channel_Details, ready to be sent over D-Bus. Free
+ * with _mcd_channel_details_free().
+ */
+GPtrArray *
+_mcd_channel_details_build_from_list (GList *channels)
+{
+    GPtrArray *channel_array;
+    GList *list;
+
+    channel_array = g_ptr_array_sized_new (g_list_length (channels));
+    for (list = channels; list != NULL; list = list->next)
+    {
+        McdChannel *channel = MCD_CHANNEL (list->data);
+        GHashTable *properties;
+        GValue channel_val = { 0, };
+        GType type;
+
+        properties = _mcd_channel_get_immutable_properties (channel);
+
+        type = TP_STRUCT_TYPE_CHANNEL_DETAILS;
+        g_value_init (&channel_val, type);
+        g_value_take_boxed (&channel_val,
+                            dbus_g_type_specialized_construct (type));
+        dbus_g_type_struct_set (&channel_val,
+                                0, mcd_channel_get_object_path (channel),
+                                1, properties,
+                                G_MAXUINT);
+
+        g_ptr_array_add (channel_array, g_value_get_boxed (&channel_val));
+    }
+
+    return channel_array;
+}
+
+/*
+ * _mcd_channel_details_free:
+ * @channels: a #GPtrArray of Channel_Details.
+ *
+ * Frees the memory used by @channels.
+ */
+void
+_mcd_channel_details_free (GPtrArray *channels)
+{
+    GValue value = { 0, };
+
+    /* to free the array, put it into a GValue */
+    g_value_init (&value, TP_ARRAY_TYPE_CHANNEL_DETAILS_LIST);
+    g_value_take_boxed (&value, channels);
+    g_value_unset (&value);
+}
+
 /*
  * _mcd_channel_get_target_id:
  * @channel: the #McdChannel.
@@ -1139,7 +1230,6 @@ mcd_channel_new_request (McdAccount *account,
     channel->priv->request_data = crd;
     channel->priv->satisfied_requests = g_list_prepend (NULL,
                                                         g_strdup (crd->path));
-    channel->priv->latest_request_time = user_time;
 
     _mcd_channel_set_status (channel, MCD_CHANNEL_STATUS_REQUEST);
 
@@ -1189,22 +1279,15 @@ _mcd_channel_get_request_path (McdChannel *channel)
 /*
  * _mcd_channel_get_satisfied_requests:
  * @channel: the #McdChannel.
- * @get_latest_time: if not %NULL, the most recent request time will be copied
- *  through this pointer
  *
  * Returns: a list of the object paths of the channel requests satisfied by
  * this channel, if the channel status is not yet MCD_CHANNEL_STATUS_DISPATCHED
  * or MCD_CHANNEL_STATUS_FAILED.
  */
 const GList *
-_mcd_channel_get_satisfied_requests (McdChannel *channel,
-                                     gint64 *get_latest_time)
+_mcd_channel_get_satisfied_requests (McdChannel *channel)
 {
     g_return_val_if_fail (MCD_IS_CHANNEL (channel), NULL);
-
-    if (get_latest_time != NULL)
-        *get_latest_time = channel->priv->latest_request_time;
-
     return channel->priv->satisfied_requests;
 }
 
@@ -1355,10 +1438,6 @@ _mcd_channel_set_request_proxy (McdChannel *channel, McdChannel *source)
     request_path = _mcd_channel_get_request_path (channel);
     if (G_LIKELY (request_path))
     {
-        source->priv->latest_request_time =
-            MAX (source->priv->latest_request_time,
-                 channel->priv->latest_request_time);
-
         source->priv->satisfied_requests =
             g_list_prepend (source->priv->satisfied_requests,
                             g_strdup (request_path));
@@ -1433,9 +1512,9 @@ channel_request_proceed (TpSvcChannelRequest *iface,
     }
 
     self->priv->request_data->proceeding = TRUE;
-    tp_svc_channel_request_return_from_proceed (context);
     _mcd_account_proceed_with_request (self->priv->request_data->account,
                                        self);
+    tp_svc_channel_request_return_from_proceed (context);
 }
 
 gboolean
@@ -1592,7 +1671,7 @@ _mcd_channel_depart (McdChannel *channel,
 
     d = g_slice_new (DepartData);
     d->reason = reason;
-    d->message = g_strdup (message);
+    d->message = g_strdup (message == NULL ? "" : message);
 
     tp_channel_call_when_ready (channel->priv->tp_chan,
                                 mcd_channel_ready_to_depart_cb, d);
