@@ -75,9 +75,7 @@
 #include "_gen/cli-Connection_Interface_Contact_Capabilities_Draft1-body.h"
 #include "_gen/cli-Connection_Interface_Contact_Capabilities-body.h"
 
-#define INITIAL_RECONNECTION_TIME   3 /* seconds */
-#define RECONNECTION_MULTIPLIER     3
-#define MAXIMUM_RECONNECTION_TIME   30 * 60 /* half an hour */
+#define INITIAL_RECONNECTION_TIME   1 /* 1 second */
 
 #define MCD_CONNECTION_PRIV(mcdconn) (MCD_CONNECTION (mcdconn)->priv)
 
@@ -124,6 +122,7 @@ struct _McdConnectionPrivate
     guint has_capabilities_if : 1;
     guint has_contact_capabilities_draft1_if : 1;
     guint has_contact_capabilities_if : 1;
+    guint has_requests_if : 1;
 
     /* FALSE until the dispatcher has said it's ready for us */
     guint dispatching_started : 1;
@@ -140,9 +139,6 @@ struct _McdConnectionPrivate
     /* FALSE until mcd_connection_close() is called */
     guint closed : 1;
 
-    /* FALSE until connected and the supported presence statuses retrieved */
-    guint presence_info_ready : 1;
-
     gchar *alias;
 
     gboolean is_disposed;
@@ -205,8 +201,6 @@ presence_set_status_cb (TpConnection *proxy, const GError *error,
 
     if (error)
     {
-        _mcd_account_set_changing_presence (priv->account, FALSE);
-
         g_warning ("%s: Setting presence of %s failed: %s",
 		   G_STRFUNC, mcd_account_get_unique_name (priv->account),
                    error->message);
@@ -318,24 +312,9 @@ _mcd_connection_set_presence (McdConnection * connection,
 
     if (_check_presence (priv, presence, &adj_status))
     {
-        TpConnectionPresenceType curr_presence;
-        const gchar *curr_status;
-        const gchar *curr_message;
-
         DEBUG ("Setting status '%s' of type %u ('%s' was requested)",
                adj_status, presence, status);
 
-        mcd_account_get_current_presence (priv->account, &curr_presence,
-                                          &curr_status, &curr_message);
-        if (curr_presence == presence &&
-            tp_strdiff (curr_status, adj_status) == 0 &&
-            tp_strdiff (curr_message, message) == 0)
-        {
-            // PresencesChanged won't be emitted and Account.ChangingPresence
-            // will never go to FALSE, so forcibly set it to FALSE
-            _mcd_account_set_changing_presence (priv->account, FALSE);
-        }
-
         tp_cli_connection_interface_simple_presence_call_set_presence
             (priv->tp_conn, -1, adj_status, message, presence_set_status_cb,
              priv, NULL, (GObject *)connection);
@@ -413,11 +392,6 @@ presence_get_statuses_cb (TpProxy *proxy, const GValue *v_statuses,
     /* Now the presence info is ready. We can set the presence */
     mcd_account_get_requested_presence (priv->account, &presence,
                                         &status, &message);
-    if (priv->connected)
-    {
-        priv->presence_info_ready = TRUE;
-    }
-
     _mcd_connection_set_presence (connection, presence, status, message);
 }
 
@@ -564,6 +538,12 @@ on_new_channel (TpConnection *proxy, const gchar *chan_obj_path,
            chan_obj_path, chan_type, handle_type, handle,
            suppress_handler ? 'T' : 'F');
 
+    /* ignore all our own requests (they have always suppress_handler = 1) as
+     * well as other requests for which our intervention has not been requested
+     * */
+    if (suppress_handler) return;
+
+    /* It's an incoming channel, so we create a new McdChannel for it */
     if (priv->dispatched_initial_channels)
     {
         channel = mcd_channel_new_from_path (proxy,
@@ -572,16 +552,10 @@ on_new_channel (TpConnection *proxy, const gchar *chan_obj_path,
         if (G_UNLIKELY (!channel)) return;
         mcd_operation_take_mission (MCD_OPERATION (connection),
                                     MCD_MISSION (channel));
-
-        /* MC no longer calls RequestChannel. As a result, if suppress_handler
-         * is TRUE, we know that this channel was requested "behind our back",
-         * therefore we should call ObserveChannels, but refrain from calling
-         * AddDispatchOperation or HandleChannels.
-         *
-         * We assume that channels without suppress_handler are incoming. */
+        /* Dispatch the incoming channel */
         _mcd_dispatcher_take_channels (priv->dispatcher,
                                        g_list_prepend (NULL, channel),
-                                       suppress_handler, suppress_handler);
+                                       FALSE);
     }
 }
 
@@ -612,6 +586,7 @@ _mcd_connection_setup_capabilities (McdConnection *connection)
     McdConnectionPrivate *priv = MCD_CONNECTION_PRIV (connection);
     GPtrArray *capabilities;
     const gchar *removed = NULL;
+    const gchar *protocol_name;
     GType type;
     guint i;
 
@@ -626,7 +601,9 @@ _mcd_connection_setup_capabilities (McdConnection *connection)
         DEBUG ("connection does not support capabilities interface");
 	return;
     }
-    capabilities = _mcd_dispatcher_get_channel_capabilities (priv->dispatcher);
+    protocol_name = mcd_account_get_protocol_name (priv->account);
+    capabilities = _mcd_dispatcher_get_channel_capabilities (priv->dispatcher,
+                                                             protocol_name);
     DEBUG ("advertising capabilities");
     tp_cli_connection_interface_capabilities_call_advertise_capabilities (priv->tp_conn, -1,
 									  capabilities,
@@ -1059,7 +1036,7 @@ on_connection_status_changed (TpConnection *tp_conn, GParamSpec *pspec,
     {
     case TP_CONNECTION_STATUS_CONNECTING:
         g_signal_emit (connection, signals[CONNECTION_STATUS_CHANGED], 0,
-                       conn_status, conn_reason, tp_conn);
+                       conn_status, conn_reason);
         priv->abort_reason = TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED;
         priv->connected = FALSE;
         break;
@@ -1067,7 +1044,7 @@ on_connection_status_changed (TpConnection *tp_conn, GParamSpec *pspec,
     case TP_CONNECTION_STATUS_CONNECTED:
         {
             g_signal_emit (connection, signals[CONNECTION_STATUS_CHANGED], 0,
-                           conn_status, conn_reason, tp_conn);
+                           conn_status, conn_reason);
 
             if (priv->probation_timer == 0)
             {
@@ -1137,15 +1114,14 @@ mcd_connection_invalidated_cb (TpConnection *tp_conn,
          * abort the connection but try to reconnect later */
         if (priv->reconnect_timer == 0)
         {
-            DEBUG ("Preparing for reconnection in %u seconds",
-                priv->reconnect_interval);
+            DEBUG ("Preparing for reconnection");
             priv->reconnect_timer = g_timeout_add_seconds
                 (priv->reconnect_interval,
                  (GSourceFunc)mcd_connection_reconnect, connection);
-            priv->reconnect_interval *= RECONNECTION_MULTIPLIER;
-
-            if (priv->reconnect_interval >= MAXIMUM_RECONNECTION_TIME)
-                priv->reconnect_interval = MAXIMUM_RECONNECTION_TIME;
+            priv->reconnect_interval *= 2;
+            if (priv->reconnect_interval >= 30 * 60)
+                /* no more than 30 minutes! */
+                priv->reconnect_interval = 30 * 60;
         }
     }
     else
@@ -1226,7 +1202,6 @@ on_new_channels (TpConnection *proxy, const GPtrArray *channels,
     McdChannel *channel;
     GList *channel_list = NULL;
     gboolean requested = FALSE;
-    gboolean only_observe = FALSE;
     guint i;
 
     if (DEBUGGING)
@@ -1257,8 +1232,10 @@ on_new_channels (TpConnection *proxy, const GPtrArray *channels,
      * FALSE: they'll also be in Channels in the GetAll(Requests) result */
     if (!priv->dispatched_initial_channels) return;
 
-    only_observe = ! MCD_CONNECTION_GET_CLASS (connection)->need_dispatch (
-        connection, channels);
+    /* first, check if we have to dispatch the channels at all */
+    if (!MCD_CONNECTION_GET_CLASS (connection)->need_dispatch (connection,
+                                                               channels))
+        return;
 
     sp_timestamp ("NewChannels received");
     for (i = 0; i < channels->len; i++)
@@ -1293,14 +1270,7 @@ on_new_channels (TpConnection *proxy, const GPtrArray *channels,
         channel_list = g_list_prepend (channel_list, channel);
     }
 
-    if (!requested)
-    {
-        /* we always dispatch unrequested (incoming) channels */
-        only_observe = FALSE;
-    }
-
-    _mcd_dispatcher_take_channels (priv->dispatcher, channel_list, requested,
-                                   only_observe);
+    _mcd_dispatcher_take_channels (priv->dispatcher, channel_list, requested);
 }
 
 static void
@@ -1319,8 +1289,7 @@ mcd_connection_recover_channel (McdConnection *connection,
     mcd_operation_take_mission (MCD_OPERATION (connection),
                                 MCD_MISSION (channel));
 
-    _mcd_dispatcher_recover_channel (priv->dispatcher, channel,
-      mcd_account_get_object_path (priv->account));
+    _mcd_dispatcher_recover_channel (priv->dispatcher, channel);
 }
 
 static void
@@ -1544,6 +1513,8 @@ on_connection_ready (TpConnection *tp_conn, const GError *error,
         MC_IFACE_QUARK_CONNECTION_INTERFACE_CONTACT_CAPABILITIES_DRAFT1);
     priv->has_contact_capabilities_if = tp_proxy_has_interface_by_id (tp_conn,
         TP_IFACE_QUARK_CONNECTION_INTERFACE_CONTACT_CAPABILITIES);
+    priv->has_requests_if = tp_proxy_has_interface_by_id (tp_conn,
+        TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS);
 
     if (priv->has_presence_if)
 	_mcd_connection_setup_presence (connection);
@@ -1560,10 +1531,7 @@ on_connection_ready (TpConnection *tp_conn, const GError *error,
     if (priv->has_alias_if)
 	_mcd_connection_setup_alias (connection);
 
-    if (!priv->dispatching_started)
-        _mcd_dispatcher_add_connection (priv->dispatcher, connection);
-
-    request_unrequested_channels (connection);
+    _mcd_dispatcher_add_connection (priv->dispatcher, connection);
 
     g_signal_emit (connection, signals[READY], 0);
 }
@@ -1579,15 +1547,15 @@ _mcd_connection_start_dispatching (McdConnection *self,
 
     self->priv->dispatching_started = TRUE;
 
-    if (tp_proxy_has_interface_by_id (self->priv->tp_conn,
-            TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS))
+    if (self->priv->has_requests_if)
         mcd_connection_setup_requests (self);
     else
         mcd_connection_setup_pre_requests (self);
 
-    /* FIXME: why is this here? if we need to update caps before and after   *
-     * connected, it should be in the call_when_ready callback.              */
     _mcd_connection_update_client_caps (self, client_caps);
+
+    /* and request all channels */
+    request_unrequested_channels (self);
 }
 
 void
@@ -1617,12 +1585,6 @@ mcd_connection_done_task_before_connect (McdConnection *self)
             DEBUG ("TpConnection went away, not doing anything");
         }
 
-        if (tp_proxy_has_interface_by_id (self->priv->tp_conn,
-                TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS))
-        {
-            _mcd_dispatcher_add_connection (self->priv->dispatcher, self);
-        }
-
         DEBUG ("%s: Calling Connect()",
                tp_proxy_get_object_path (self->priv->tp_conn));
         tp_cli_connection_call_connect (self->priv->tp_conn, -1, connect_cb,
@@ -1742,12 +1704,6 @@ mcd_connection_early_get_interfaces_cb (TpConnection *tp_conn,
                  * we can't usefully pre-load capabilities - we'll be told
                  * the real capabilities as soon as it has worked them out */
             }
-            else if (q == TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS)
-            {
-              /* If we have the Requests iface, we could start dispatching
-               * before the connection is in CONNECTED state */
-              tp_proxy_add_interface_by_id ((TpProxy *) tp_conn, q);
-            }
         }
     }
 
@@ -1823,7 +1779,7 @@ request_connection_cb (TpConnectionManager *proxy, const gchar *bus_name,
         {
             g_signal_emit (connection, signals[CONNECTION_STATUS_CHANGED], 0,
                            TP_CONNECTION_STATUS_DISCONNECTED,
-                           TP_CONNECTION_STATUS_REASON_REQUESTED, NULL);
+                           TP_CONNECTION_STATUS_REASON_REQUESTED);
         }
 
         return;
@@ -1838,7 +1794,7 @@ request_connection_cb (TpConnectionManager *proxy, const gchar *bus_name,
 
         g_signal_emit (connection, signals[CONNECTION_STATUS_CHANGED], 0,
             TP_CONNECTION_STATUS_DISCONNECTED,
-            TP_CONNECTION_STATUS_REASON_NETWORK_ERROR, NULL);
+            TP_CONNECTION_STATUS_REASON_NETWORK_ERROR);
         return;
     }
 
@@ -1875,7 +1831,7 @@ _mcd_connection_connect_with_params (McdConnection *connection,
 
     g_signal_emit (connection, signals[CONNECTION_STATUS_CHANGED], 0,
                    TP_CONNECTION_STATUS_CONNECTING,
-                   TP_CONNECTION_STATUS_REASON_REQUESTED, NULL);
+                   TP_CONNECTION_STATUS_REASON_REQUESTED);
 
     /* If the McdConnection gets aborted (which results in it being freed!),
      * we need to kill off the Connection. So, we can't use connection as the
@@ -1911,11 +1867,9 @@ _mcd_connection_release_tp_connection (McdConnection *connection)
     DEBUG ("%p", connection);
     g_signal_emit (connection, signals[SELF_PRESENCE_CHANGED], 0,
                    TP_CONNECTION_PRESENCE_TYPE_OFFLINE, "offline", "");
-
     g_signal_emit (connection, signals[CONNECTION_STATUS_CHANGED], 0,
                    TP_CONNECTION_STATUS_DISCONNECTED,
-                   priv->abort_reason, priv->tp_conn);
-
+                   priv->abort_reason);
     if (priv->tp_conn)
     {
 	/* Disconnect signals */
@@ -1928,6 +1882,7 @@ _mcd_connection_release_tp_connection (McdConnection *connection)
 	_mcd_connection_call_disconnect (connection);
 	g_object_unref (priv->tp_conn);
 	priv->tp_conn = NULL;
+	_mcd_account_tp_connection_changed (priv->account);
     }
 
     /* the interface proxies obtained from this connection must be deleted, too
@@ -2157,11 +2112,8 @@ _mcd_connection_request_channel (McdConnection *connection,
         return TRUE;
     }
 
-    if (tp_proxy_has_interface_by_id (priv->tp_conn,
-            TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS))
-    {
+    if (priv->has_requests_if)
         ret = request_channel_new_iface (connection, channel);
-    }
     else
     {
         mcd_channel_take_error (channel,
@@ -2205,7 +2157,7 @@ mcd_connection_class_init (McdConnectionClass * klass)
     klass->need_dispatch = mcd_connection_need_dispatch;
     klass->request_channel = _mcd_connection_request_channel;
 
-    _mcd_ext_register_dbus_glib_marshallers ();
+    _mc_ext_register_dbus_glib_marshallers ();
 
     tp_connection_init_known_interfaces ();
     tp_proxy_or_subclass_hook_on_interface_add (TP_TYPE_CONNECTION,
@@ -2261,8 +2213,8 @@ mcd_connection_class_init (McdConnectionClass * klass)
     signals[CONNECTION_STATUS_CHANGED] = g_signal_new (
         "connection-status-changed", G_OBJECT_CLASS_TYPE (klass),
         G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED, 0,
-        NULL, NULL, _mcd_marshal_VOID__UINT_UINT_OBJECT,
-        G_TYPE_NONE, 3, G_TYPE_UINT, G_TYPE_UINT, TP_TYPE_CONNECTION);
+        NULL, NULL, _mcd_marshal_VOID__UINT_UINT,
+        G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_UINT);
 
     signals[READY] = g_signal_new ("ready",
         G_OBJECT_CLASS_TYPE (klass),
@@ -2408,16 +2360,14 @@ request_channel_new_iface (McdConnection *connection, McdChannel *channel)
     properties = _mcd_channel_get_requested_properties (channel);
     if (_mcd_channel_get_request_use_existing (channel))
     {
-        /* Timeout of 5 hours: 5 * 3600 * 1000 */
         tp_cli_connection_interface_requests_call_ensure_channel
-            (priv->tp_conn, 18000000, properties, ensure_channel_cb,
+            (priv->tp_conn, -1, properties, ensure_channel_cb,
              connection, NULL, (GObject *)channel);
     }
     else
     {
-        /* Timeout of 5 hours: 5 * 3600 * 1000 */
         tp_cli_connection_interface_requests_call_create_channel
-            (priv->tp_conn, 18000000, properties, create_channel_cb,
+            (priv->tp_conn, -1, properties, create_channel_cb,
              connection, NULL, (GObject *)channel);
     }
     return TRUE;
@@ -2615,13 +2565,10 @@ _mcd_connection_set_tp_connection (McdConnection *connection,
     {
         g_signal_emit (connection, signals[CONNECTION_STATUS_CHANGED], 0,
             TP_CONNECTION_STATUS_DISCONNECTED,
-            TP_CONNECTION_STATUS_REASON_NETWORK_ERROR,
-            NULL);
+            TP_CONNECTION_STATUS_REASON_NETWORK_ERROR);
         return;
     }
-    /* FIXME: need some way to feed the status into the Account, but we don't
-     * actually know it yet */
-    _mcd_account_tp_connection_changed (priv->account, priv->tp_conn);
+    _mcd_account_tp_connection_changed (priv->account);
 
     /* Setup signals */
     g_signal_connect (priv->tp_conn, "invalidated",
@@ -2660,11 +2607,3 @@ _mcd_connection_is_ready (McdConnection *self)
     return (self->priv->tp_conn != NULL) &&
         tp_connection_is_ready (self->priv->tp_conn);
 }
-
-gboolean
-_mcd_connection_presence_info_is_ready (McdConnection *self)
-{
-    g_return_val_if_fail (MCD_IS_CONNECTION (self), FALSE);
-
-    return self->priv->presence_info_ready;
-}
