@@ -22,6 +22,7 @@ Infrastructure code for testing Mission Control
 
 import base64
 import os
+import sha
 import sys
 
 import constants as cs
@@ -406,10 +407,6 @@ class SimulatedConnection(object):
     def GetStatus(self, e):
         self.q.dbus_return(e.message, self.status, signature='u')
 
-    def ConnectionError(self, error, details):
-        self.q.dbus_emit(self.object_path, cs.CONN, 'ConnectionError',
-                error, details, signature='sa{sv}')
-
     def StatusChanged(self, status, reason):
         self.status = status
         self.reason = reason
@@ -593,9 +590,8 @@ def aasv(x):
 class SimulatedClient(object):
     def __init__(self, q, bus, clientname,
             observe=[], approve=[], handle=[],
-            cap_tokens=[], bypass_approval=False, wants_recovery=False,
-            request_notification=True, implement_get_interfaces=True,
-            is_handler=None):
+            cap_tokens=[], bypass_approval=False,
+            request_notification=True, implement_get_interfaces=True):
         self.q = q
         self.bus = bus
         self.bus_name = '.'.join([cs.tp_name_prefix, 'Client', clientname])
@@ -605,14 +601,9 @@ class SimulatedClient(object):
         self.approve = aasv(approve)
         self.handle = aasv(handle)
         self.bypass_approval = bool(bypass_approval)
-        self.wants_recovery = bool(wants_recovery)
         self.request_notification = bool(request_notification)
         self.handled_channels = dbus.Array([], signature='o')
         self.cap_tokens = dbus.Array(cap_tokens, signature='s')
-        self.is_handler = is_handler
-
-        if self.is_handler is None:
-            self.is_handler = bool(handle)
 
         if implement_get_interfaces:
             q.add_dbus_method_impl(self.Get_Interfaces,
@@ -651,9 +642,6 @@ class SimulatedClient(object):
         q.add_dbus_method_impl(self.Get_BypassApproval,
                 path=self.object_path, interface=cs.PROPERTIES_IFACE,
                 method='Get', args=[cs.HANDLER, 'BypassApproval'])
-        q.add_dbus_method_impl(self.Get_Recover,
-                path=self.object_path, interface=cs.PROPERTIES_IFACE,
-                method='Get', args=[cs.OBSERVER, 'Recover'])
         q.add_dbus_method_impl(self.GetAll_Handler,
                 path=self.object_path,
                 interface=cs.PROPERTIES_IFACE, method='GetAll',
@@ -662,9 +650,6 @@ class SimulatedClient(object):
     def release_name(self):
         del self._bus_name_ref
 
-    def reacquire_name(self):
-        self._bus_name_ref = dbus.service.BusName(self.bus_name, self.bus)
-
     def get_interfaces(self):
         ret = dbus.Array([], signature='s', variant_level=1)
 
@@ -674,7 +659,7 @@ class SimulatedClient(object):
         if self.approve:
             ret.append(cs.APPROVER)
 
-        if self.is_handler:
+        if self.handle:
             ret.append(cs.HANDLER)
 
         if self.request_notification:
@@ -692,10 +677,7 @@ class SimulatedClient(object):
 
     def GetAll_Observer(self, e):
         assert self.observe
-        self.q.dbus_return(e.message, {
-            'ObserverChannelFilter': self.observe,
-            'Recover': self.wants_recovery,
-            },
+        self.q.dbus_return(e.message, {'ObserverChannelFilter': self.observe},
                 signature='a{sv}', bus=self.bus)
 
     def Get_ObserverChannelFilter(self, e):
@@ -714,7 +696,7 @@ class SimulatedClient(object):
                 bus=self.bus)
 
     def GetAll_Handler(self, e):
-        assert self.is_handler
+        assert self.handle
         self.q.dbus_return(e.message, {
             'HandlerChannelFilter': self.handle,
             'BypassApproval': self.bypass_approval,
@@ -741,11 +723,6 @@ class SimulatedClient(object):
         self.q.dbus_return(e.message, self.bypass_approval, signature='v',
                 bus=self.bus)
 
-    def Get_Recover(self, e):
-        assert self.handle
-        self.q.dbus_return(e.message, self.recover, signature='v',
-                bus=self.bus)
-
 def create_fakecm_account(q, bus, mc, params):
     """Create a fake connection manager and an account that uses it.
     """
@@ -828,17 +805,8 @@ def enable_fakecm_account(q, bus, mc, account, expected_params,
 
     q.dbus_return(e.message, conn.bus_name, conn.object_path, signature='so')
 
-    if has_requests:
-        expect_before_connect.append(
-                servicetest.EventPattern('dbus-method-call',
-                    interface=cs.PROPERTIES_IFACE, method='GetAll',
-                    args=[cs.CONN_IFACE_REQUESTS],
-                    path=conn.object_path, handled=True))
-
     if expect_before_connect:
         events = list(q.expect_many(*expect_before_connect))
-        if has_requests:
-            del events[-1]
     else:
         events = []
 
@@ -848,7 +816,13 @@ def enable_fakecm_account(q, bus, mc, account, expected_params,
 
     expect_after_connect = list(expect_after_connect)
 
-    if not has_requests:
+    if has_requests:
+        expect_after_connect.append(
+                servicetest.EventPattern('dbus-method-call',
+                    interface=cs.PROPERTIES_IFACE, method='GetAll',
+                    args=[cs.CONN_IFACE_REQUESTS],
+                    path=conn.object_path, handled=True))
+    else:
         expect_after_connect.append(
                 servicetest.EventPattern('dbus-method-call',
                     interface=cs.CONN, method='ListChannels', args=[],
@@ -856,8 +830,7 @@ def enable_fakecm_account(q, bus, mc, account, expected_params,
 
     events = events + list(q.expect_many(*expect_after_connect))
 
-    if not has_requests:
-        del events[-1]
+    del events[-1]
 
     if events:
         return (conn,) + tuple(events)
